/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public 
 * License version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Initial Developer of the Original Code is R.V.Guha. Portions
 * created by R.V.Guha are copyright (c) 2000 R.V.Guha. All Rights
 * Reserved.
 */

/* This file implements the http and file protocols to access data to feed the
   parser. The parser is hungry.

   On linux, I am using the ghttp library. Need to find corresponding libraries
   on other unix platforms.

 */
  

#include <ghttp.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "contentSource.h"
#include <stdio.h>
#include <string.h>
#include "../utils/utils.h"

typedef enum {
  HTTP,
  FILEP
} PROTOCOL;

typedef struct _ContentSourceStruct {
  PROTOCOL protocol;
  ghttp_request* httpHandler;
  FILE    *fileHandler;
  char    *buff;
  size_t  size;
  size_t  offset;
} _ContentSourceStruct;

int
validURL (char* url) {
  return (strstr(url, "http://") || strstr(url, "file:/"));
}

PROTOCOL
protocolOfURL (char* url) {
  if (strncmp(url, "http://", 7) == 0) return HTTP;
  if (strncmp(url, "file:/", 6) == 0) return FILEP;
  return -1;
}

ContentSource 
CS_NewContentSource(char* url) {
  if (validURL(url)) {
    ContentSource cs = (ContentSource) getMem(sizeof(_ContentSourceStruct));
    cs->protocol = protocolOfURL(url);
    cs->size = 0;
    cs->offset = 0;
    if (cs->protocol == HTTP) {
      ghttp_request* request = ghttp_request_new();
      cs->httpHandler = request;
      ghttp_set_uri(request, url);
      ghttp_set_header(request, http_hdr_Connection, "close");
      /*   ghttp_set_sync(request,   ghttp_async); */
      ghttp_prepare(request);
      return cs;
    } else if (cs->protocol == FILEP) {
      char* start = strstr(url, ":/");
      struct stat st;
      start = start + 1;
      /*      if (start[0] == '/') start++;
              if (start[0] == '/') start++; */

      if (stat(start, &st)) {
        freeMem((void*)cs);
        return 0;
      } else {
        cs->size = st.st_size;
      }

      cs->fileHandler = fopen(start, "r");
      if (cs->fileHandler) {        
        return cs;
      } else {
        freeMem((void*)cs);
        return 0;
      }
    } else return 0;
  }
}
        	
CS_Status 
CS_Start(ContentSource cs) {
  if (!cs) return CS_ERROR;
  if (cs->protocol == HTTP) {
    ghttp_process(cs->httpHandler);
  } 
  return CS_OK;
}
  

CS_Status 
CS_GetContent(ContentSource cs, char* body, size_t bufferSize, size_t* len) {
  if (!cs) return CS_ERROR;
  if (cs->protocol == HTTP) {
    if (cs->size) {
      size_t remaining = cs->size - cs->offset;
      size_t amount = (bufferSize < remaining ? bufferSize : remaining);
      if (remaining < 1) return CS_DONE;
      cs->offset = cs->offset + remaining;
      memcpy(body, cs->buff, amount);
      *len = amount;
      return CS_OK;
    } else {
      ghttp_status status = ghttp_status_code(cs->httpHandler);
      printf("status=%i\n", status);
      if (status == 200) {
        cs->buff = ghttp_get_body(cs->httpHandler);
        cs->size  = ghttp_get_body_len(cs->httpHandler);
        *len = 0;
        return CS_PROCESSING;      
      } else if (status != ghttp_not_done) {
        return CS_ERROR;
      } else return CS_PROCESSING;  
    }
  } else if (cs->protocol == FILEP) {
    if (feof(cs->fileHandler)) return CS_DONE;
    *len = fread(body, bufferSize, 1, cs->fileHandler);
    return CS_OK;
  } else {
    return CS_ERROR;
  }
}
 
CS_Status
CS_Close(ContentSource cs) {
  if (!cs) return CS_ERROR;
  if (cs->protocol == HTTP) {
    ghttp_request_destroy(cs->httpHandler);
    freeMem((void*)cs);
    return CS_OK;
  } else if (cs->protocol == FILEP) {
    fclose(cs->fileHandler);    
    freeMem((void*)cs);
    return CS_OK;
  } else return CS_ERROR;
}
    
