/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public 
 * License version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Initial Developer of the Original Code is R.V.Guha. Portions
 * created by R.V.Guha are copyright (c) 2000 R.V.Guha. All Rights
 * Reserved.
 */

#include "../utils/utils.h"
#include "rdf-int.h"
#include "indexPool.h"

#define INDEX_POOL_INCREMENT 128
#define INDEX_POOL_SIZE 128

pthread_mutex_t gIndexPoolMutex = PTHREAD_MUTEX_INITIALIZER;
pthread_mutex_t gCursorPoolMutex = PTHREAD_MUTEX_INITIALIZER;

static Index gIndexPool[INDEX_POOL_SIZE] ;

void
initIndexPool () {
  memset(gIndexPool, 0, sizeof(gIndexPool));
}

Index
getIndexForLen (size_t len) {
  int offset = len / INDEX_POOL_INCREMENT;
  Index existing ;
  if (len < (INDEX_POOL_SIZE * INDEX_POOL_INCREMENT)) {
    pthread_mutex_lock(&gIndexPoolMutex);
    existing = gIndexPool[offset];
    if (!existing) {
      pthread_mutex_unlock(&gIndexPoolMutex);
      existing = (Index)getMem(sizeof(IndexStruct));
      existing->offset = offset;
      existing->maxLen = (offset+1) * INDEX_POOL_INCREMENT;
      existing->data = (char*)getMem(existing->maxLen);
      return existing;
    } else {
      gIndexPool[offset] = existing->next;
      existing->next = 0;
      pthread_mutex_unlock(&gIndexPoolMutex);
      bzero(existing->data, existing->maxLen);
      existing->free = 0;
      return existing;
    }
  } else return 0;
}

void
freeIndex (Index ind) {
  if (!ind->free) {
    pthread_mutex_lock(&gIndexPoolMutex);
    ind->free = 1;
    ind->next = gIndexPool[ind->offset];
    gIndexPool[ind->offset] = ind;
    pthread_mutex_unlock(&gIndexPoolMutex);
  } else {
    printf("Warning! Multiple Index Frees!\n");
  }
}
  
static RDF_Cursor gCursorPool = 0;

RDF_Cursor
getNewCursor () {
  RDF_Cursor existing;
  pthread_mutex_lock(&gCursorPoolMutex);
  existing = gCursorPool;
  if (existing) {
    gCursorPool = existing->next;
  } 
  pthread_mutex_unlock(&gCursorPoolMutex);
  if (!existing) {
    existing = (RDF_Cursor) getMem(sizeof(RDF_CursorStruct));
  }
  return existing;
}

void 
freeCursor (RDF_Cursor c) {
  pthread_mutex_lock(&gCursorPoolMutex);
  c->next = gCursorPool;
  gCursorPool = c;
  pthread_mutex_unlock(&gCursorPoolMutex);
}
  
  
