/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public 
 * License version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Initial Developer of the Original Code is R.V.Guha. Portions
 * created by R.V.Guha are copyright (c) 2000 R.V.Guha. All Rights
 * Reserved.
 */

/* This file implements the http and file protocols to access data to feed the
   parser. The parser is hungry.

 */
  


#include <sys/types.h>
#include <sys/stat.h>
#include "contentSource.h"
#include <stdio.h>
#include <string.h>
#include "../utils/utils.h"
#include	<netdb.h>		/* for struct hostent */
#include	<sys/socket.h>		/* for AF_INET */
#include	<netinet/in.h>		/* for struct in_addr */
#include	<arpa/inet.h>		/* for inet_ntoa() */



typedef enum {
  HTTP,
  FILEP
} PROTOCOL;

typedef struct _ContentSourceStruct {
  PROTOCOL protocol;
  char* path;
  char* host;
  FILE    *fileHandler;
  size_t  size;
  size_t  offset;
  int socket;
} _ContentSourceStruct;

int
validURL (char* url) {
  return (strstr(url, "http://") || strstr(url, "file:/"));
}

PROTOCOL
protocolOfURL (char* url) {
  if (strncmp(url, "http://", 7) == 0) return HTTP;
  if (strncmp(url, "file:/", 6) == 0) return FILEP;
  return -1;
}

ContentSource 
CS_NewContentSource(char* url) {
  if (validURL(url)) {
    ContentSource cs = (ContentSource) getMem(sizeof(_ContentSourceStruct));
    cs->protocol = protocolOfURL(url);
    cs->size = 0;
    cs->offset = 0;
    if (cs->protocol == HTTP) {
      char* rest = &url[7];
      char* path = strchr(rest, '/');
      cs->host = getMem(path-rest+1);
      memcpy(cs->host, rest, path-rest);
      cs->path = path;
      return cs;
    } else if (cs->protocol == FILEP) {
      char* start = strstr(url, ":/");
      struct stat st;
      start = start + 1;
      if (strncmp(start, "///", 3) == 0) {
        start = start + 2;
      }
      /*      if (start[0] == '/') start++;
              if (start[0] == '/') start++; */

      if (stat(start, &st)) {
        freeMem((void*)cs);
        return 0;
      } else {
        cs->size = st.st_size;
      }

      cs->fileHandler = fopen(start, "r");
      if (cs->fileHandler) {        
        return cs;
      } else {
        freeMem((void*)cs);
        return 0;
      }
    } else return 0;
  }
}
        	
CS_Status 
CS_Start(ContentSource cs) {
  char buff[512];
  if (!cs) return CS_ERROR;
  if (cs->protocol == HTTP) {
    int			sockfd;
    struct sockaddr_in	serv_addr;
    struct hostent	*hostptr;
    struct in_addr	*ptr;
    
    if ((hostptr = gethostbyname(cs->host)) == NULL) {
      printf("gethostbyname error for host: %s ",
             cs->host);
      return CS_ERROR;
    }
  
    bzero((char *) &serv_addr, sizeof(serv_addr));
    serv_addr.sin_family      = AF_INET;
    ptr = (struct in_addr *)*(hostptr->h_addr_list);
    serv_addr.sin_addr.s_addr = ptr->s_addr ;
    serv_addr.sin_port        = htons(80);
    
    if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
      printf("client: can't open stream socket");
      return CS_ERROR;
    }
    if (connect(sockfd, (struct sockaddr *) &serv_addr,
                sizeof(serv_addr)) < 0) {
      printf("client: can't connect to server");
      return CS_ERROR;
    }    

    snprintf(buff, 512, "GET %s HTTP/1.1\nHost: %s\nConnection: Close\n\n",
             cs->path, cs->host);
    writen(sockfd, buff, strlen(buff));
  
    cs->socket = sockfd;
  } 
  return CS_OK;
}
  


CS_Status 
CS_GetContent(ContentSource cs, char* body, size_t bufferSize, size_t* len) {
  if (!cs) return CS_ERROR;
  if (cs->protocol == HTTP) {
    int n = read(cs->socket, body, bufferSize);
    *len = n;
    if (n == 0) {
      return CS_DONE;
    } else if (n == -1) {
      return CS_ERROR;
    } else return CS_OK;
  } else if (cs->protocol == FILEP) {
    if (feof(cs->fileHandler)) return CS_DONE;
    *len = fread(body, 1, bufferSize,  cs->fileHandler);
    return CS_OK;
  } else {
    return CS_ERROR;
  }
}
 
CS_Status
CS_Close(ContentSource cs) {
  if (!cs) return CS_ERROR;
  if (cs->protocol == HTTP) {
    close(cs->socket);
    freeMem((void*)cs);
    return CS_OK;
  } else if (cs->protocol == FILEP) {
    fclose(cs->fileHandler);    
    freeMem((void*)cs);
    return CS_OK;
  } else return CS_ERROR;
}
    
