/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public 
 * License version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Initial Developer of the Original Code is R.V.Guha. Portions
 * created by R.V.Guha are copyright (c) 2000 R.V.Guha. All Rights
 * Reserved.
 */

#include <string.h>
#include <stdio.h>
#include "rdf-int.h"
#include "indexPool.h"
#include "../utils/utils.h"
#include "xmlparse.h"
#include "contentSource.h"

int gAssertionCount = 0;
char *getFileCode(RDF_Parser rdfp);

char* getMem (size_t len);
void  freeMem(char* mem);

int gVerbose = 2;

char* 
getIndexString(RDF_Resource res) {
  return (char*) res;
}

char* 
dbName (RDF_DB db) {
  return (char*) db;
}


Index
makeIndex (void* item, char* arc, RDF_ValueType type) {
  size_t resLen ;
  size_t arcLen; 
  Index ind ;

  if ((type == RDF_RESOURCE_TYPE) || (type == RDF_STRING_TYPE)) {
    char* res = (char*) item;
    resLen = strlen(res);
    arcLen = strlen(arc);
    ind = getIndexForLen(resLen + arcLen+1);
    memcpy(ind->data, res, resLen);
    strcat(ind->data, "|");
    strcat(ind->data, arc);
    ind->len = resLen + arcLen;
    return ind;
  } else if (type == RDF_INT_TYPE) {
    char res[12];
    snprintf(res, 12, "%i", (int) item);
    resLen = strlen(res);
    arcLen = strlen(arc);
    ind = getIndexForLen(resLen + arcLen+1);
    memcpy(ind->data, res, resLen);
    strcat(ind->data, "|");
    memcpy(&ind->data[resLen], arc, arcLen);
    ind->len = resLen + arcLen;
    return ind;
  } else {
    return 0;
  }
}


Index
makeValue (void* item,  RDF_ValueType type, char* sourceCode) {
  size_t resLen ;
  size_t sl = strlen(sourceCode);
  Index ind ;

  if ((type == RDF_RESOURCE_TYPE) || (type == RDF_STRING_TYPE)) {
    char* res = (char*) item;
    resLen = strlen(res) + 3 + sl;
    ind = getIndexForLen(resLen);
    ind->data[0] = (type == RDF_RESOURCE_TYPE ? '1' : '3');
    ind->data[1] = '\t';
    strcat(ind->data, sourceCode);
    strcat(ind->data, "\n");
    strcat(ind->data, res);
    ind->len = resLen;
    return ind;
  } else if (type == RDF_INT_TYPE) {
    char res[32];
    snprintf(res, 32, "%i\t%s\n%i", RDF_INT_TYPE, sourceCode, (int) item);
    resLen = strlen(res);
    ind = getIndexForLen(resLen);
    memcpy(ind->data, res, resLen);
    ind->len = resLen ;
    return ind;
  } else {
    return 0;
  }
}
    

int
addRemoveTriple (RDF_DB db,  RDF_Resource source, RDF_Resource arc, 
                 void* target, RDF_ValueType type, RDF_Operation op, char* sourceCode) {

  char*     realSrc  = getIndexString(source);
  char*     realArc  = getIndexString(arc);
  void*     realTarget = (type == RDF_RESOURCE_TYPE ? 
			  getIndexString((RDF_Resource) target) : target);
  
  Index forwardIndex = makeIndex(realSrc, realArc, RDF_RESOURCE_TYPE);
  Index reverseIndex = makeIndex(realTarget, realArc, type);
  Index forwardValue = makeValue(realTarget, type, sourceCode);
  Index reverseValue = makeValue(realSrc, RDF_RESOURCE_TYPE, sourceCode);
  Index arcValue     = makeValue(source,  RDF_RESOURCE_TYPE, sourceCode);
  int err = 0;

  if (op == RDF_ASSERT) {
    int err1 = DB_AddForwardIndex(db, forwardIndex->data, forwardValue->data, 1);
    if ((err1 > -1) && (err1 != 2)) {
      if (reverseIndex) 
        err = DB_AddReverseIndex(db, reverseIndex->data, reverseValue->data, 0);
      if (!err && (err1 != 1)) err = DB_AddArcIndex(db, realArc, arcValue->data, 0);
    }
  } else if (op == RDF_UNASSERT) {
    err = DB_RemoveForwardIndex(db, forwardIndex->data, forwardValue->data);
    if (!err && reverseIndex) err = DB_RemoveReverseIndex(db, reverseIndex->data, 
                                                          reverseValue->data);
    if (!err) err = DB_RemoveArcIndex(db, realArc, arcValue->data);
  }

  freeIndex(forwardIndex);
  freeIndex(reverseIndex);
  freeIndex(forwardValue);
  freeIndex(reverseValue);
  freeIndex(arcValue);

  return err;
  
}

RDF_Result 
RDF_Initialize() {
  void initIndexPool () ;
  initIndexPool () ;
  DB_Initialize();
}


RDF_Resource 
RDF_GetResource (char* uri) {
  return uri;
}

RDF_Result
RDF_GetDB (char* name, int createp, RDF_DB *db) {
  int err = DB_CreateDatabase (name);
  if (!err) {
    *db = name;
    return 0;
  } else {
    return err;
  }
}


void
printAssertion (char* prefix, RDF_DB db, RDF_Resource source, RDF_Resource arc, void* target, RDF_ValueType type) {
  printf("%s %s(%s, ", prefix, (char*)arc, (char*)source);
  if (type == RDF_RESOURCE_TYPE) {
    printf("%s)", (char*) target);
  } else if (type == RDF_STRING_TYPE) {
    printf("\'%s\')", (char*) target);
  } else if (type == RDF_INT_TYPE) {
    printf("%i)", (int) target);
  }
  fflush(stdout);
}


RDF_Result 
RDF_Assert (RDF_DB db, RDF_Resource source, RDF_Resource arc, void* target, RDF_ValueType type) {
  if (gVerbose > 1) printAssertion("\nAsserting:", db, source, arc, target, type);  
  gAssertionCount++;
  return addRemoveTriple(db, source, arc, target, type, RDF_ASSERT, "0");
}

RDF_Result 
RDF_LoadFileAssert (RDF_Parser rdfp, RDF_DB db, RDF_Resource source, RDF_Resource arc, void* target, RDF_ValueType type) {
  if (gVerbose > 2) printAssertion("\nAsserting:", db, source, arc, target, type);  
  gAssertionCount++;
  if (!addRemoveTriple(db, source, arc, target, type, 
                       RDF_ASSERT, getFileCode(rdfp)) && (rdfpGetFile(rdfp))) {
    extern FILE *rdfpGetFile(RDF_Parser rdfp);
    FILE *file = rdfpGetFile(rdfp);
    fprintf(file, "%s\t%s\t", arc, source);
    if (type == RDF_INT_TYPE) {
      char buff[12];
      fprintf(file, "%i\n", (int)target);
    } else if (type == RDF_STRING_TYPE) {
      fprintf(file, "'%s'\n", (char*)target);
    } else if (type == RDF_RESOURCE_TYPE) {
      fprintf(file, "%s\n", (char*)target);
    }
  }
  return 0;
}


RDF_Result 
RDF_Unassert (RDF_DB db, RDF_Resource source, RDF_Resource arc, void* target, RDF_ValueType type) {
  if (gVerbose > 1) printAssertion("\nUnasserting:", db, source, arc, target, type);  
  return addRemoveTriple(db, source, arc, target, type, RDF_UNASSERT, "0");
}

int 
RDF_HasAssertion(RDF_DB db, const RDF_Resource source, const  RDF_Resource arc, const void* target, RDF_ValueType type) {
  char*  realSrc     = getIndexString(source);
  char*     realArc  = getIndexString(arc);
  Index forwardIndex = makeIndex(realSrc, realArc, RDF_RESOURCE_TYPE);
  char* dbn          = dbName(db);
  int err = 0;
  DB_Cursor dbc = 0;
  char* value;

  if (0 != (err = DB_GetForwardCursor(dbn, forwardIndex->data, &dbc))) {
    return err;
  }

  if (gVerbose > 2) {
    printf("\nHas Assertion source=%s, arc=%s target=%s", source, arc, (char*)target);
    fflush(stdout);
  }

  while (0 == DB_CursorAdvance(dbc)) {
    int vtype ;
    void *value;
    DB_CursorCurrentValue(dbc , &value);
    DB_CursorCurrentValueType(dbc, &vtype);

    if (vtype == type) {
      if ((type == RDF_RESOURCE_TYPE) || (type == RDF_STRING_TYPE)) {
        if (streq(value, (char*)target)) return 1;
      } else if (type == RDF_INT_TYPE) {
        if ((int)value == (int) target) return 1;
      }
    }
  }
  
  DB_ReleaseCursor(dbc);
}

int
RDF_GetTargets (RDF_DB db, RDF_Resource source, RDF_Resource arc, RDF_Cursor *cursor) {
  char*  realSrc     = getIndexString(source);
  char*     realArc  = getIndexString(arc);
  Index forwardIndex = makeIndex(realSrc, realArc, RDF_RESOURCE_TYPE);
  char* dbn          = dbName(db);
  DB_Cursor dbc = 0;
  int       err = 0;
  RDF_Cursor c;

  if  (0 != (err  = DB_GetForwardCursor(dbn, forwardIndex->data, &dbc))) {
    return err;
  }

  if (gVerbose > 2) {
    printf("\nGet Targets source=%s, arc=%s", source, arc);
    fflush(stdout);
  }

  if (dbc) {    
    c = getNewCursor();
    c->dbCursor = dbc;
    c->db = db;
    c->res = source;
    c->arc = arc;
    c->inversep = 0;
    c->index = forwardIndex;
  }

  *cursor = c;
  return 0;
}

int
RDF_GetArcsOut (RDF_DB db, RDF_Resource source,  RDF_Cursor *cursor) {
  char*  realSrc     = getIndexString(source);
  char* dbn          = dbName(db);
  Index ind          = makeIndex(realSrc, "", RDF_RESOURCE_TYPE);
  DB_Cursor dbc = 0;
  int       err = 0;
  RDF_Cursor c;

  if  (0 != (err  = DB_GetArcsOutCursor(dbn, ind->data, &dbc))) {
    return err;
  }

  if (gVerbose > 2) {
    printf("\nGet Arcs Out =%s",  source);
    fflush(stdout);
  }

  if (dbc) {    
    c = getNewCursor();
    c->dbCursor = dbc;
    c->db = db;
    c->res = source;
    c->index = ind;
  }

  *cursor = c;
  return 0;
}

int
RDF_GetArcsIn (RDF_DB db, RDF_Resource target,  RDF_Cursor *cursor) {
  char*  realTrg     = getIndexString(target);
  char* dbn          = dbName(db);
  Index ind          = makeIndex(realTrg, "", RDF_RESOURCE_TYPE);
  DB_Cursor dbc = 0;
  int       err = 0;
  RDF_Cursor c;

  if  (0 != (err  = DB_GetArcsInCursor(dbn, ind->data, &dbc))) {
    return err;
  }

  if (gVerbose > 2) {
    printf("\nGet Arcs In =%s", target);
    fflush(stdout);
  }

  if (dbc) {    
    c = getNewCursor();
    c->dbCursor = dbc;
    c->db = db;
    c->res = target;
    c->index = ind;
  }

  *cursor = c;
  return 0;
}
 

RDF_Result 
RDF_GetArcSources (RDF_DB db, RDF_Resource arc, RDF_Cursor *cursor) {
char*   realArc  = getIndexString(arc);
  char* dbn          = dbName(db);
  DB_Cursor dbc = 0;
  int       err = 0;
  RDF_Cursor c;

  if  (0 != (err  = DB_GetArcCursor(dbn, realArc, &dbc))) {
    return err;
  }

  if (gVerbose > 2) {
    printf("\nGet Arc sources arc=%s",  realArc);
    fflush(stdout);
  }

  if (dbc) {    
    c = getNewCursor();
    c->dbCursor = dbc;
    c->db = db;
    c->arc = arc;
    c->inversep = 0;
  }

  *cursor = c;
  return 0;
}

int
RDF_GetSources(RDF_DB db, RDF_Resource target,   RDF_Resource arc, RDF_Cursor *cursor) {
  char*  realTrg     = getIndexString(target);
  char*     realArc  = getIndexString(arc);
  Index reverseIndex = makeIndex(realTrg, realArc, RDF_RESOURCE_TYPE); 
  char* dbn          = dbName(db);
  DB_Cursor dbc  = 0 ;
  int       err = 0;
  RDF_Cursor c;

  if  (0 != (err  = DB_GetReverseCursor(dbn, reverseIndex->data, &dbc))) {
    return err;
  }

  if (gVerbose > 2) {
    printf("\nGet Sources target=%s, arc=%s", target, arc);
    fflush(stdout);
  }
  
  if (dbc) {    
    c = getNewCursor();
    c->dbCursor = dbc;
    c->db = db;
    c->res = target;
    c->arc = arc;
    c->inversep = 1;
    c->index = reverseIndex;
  }

  *cursor = c;
  return 0;
}

RDF_Result
RDF_Advance (RDF_Cursor c) {
  if (!c) {
    return RDF_NO_MORE_VALUES;
  } else {
    return (RDF_Result) DB_CursorAdvance(c->dbCursor);
  }
}

RDF_Result 
RDF_CursorCurrentValue (RDF_Cursor c, void** value) {
  if (!c) {
    return RDF_NO_MORE_VALUES;
  } else {
    return DB_CursorCurrentValue(c->dbCursor, value);
  }
}

RDF_Result 
RDF_CursorCurrentValueSize (RDF_Cursor c, int *sz) {
  if (!c) {
    return RDF_NO_MORE_VALUES;
  } else {
    return DB_CursorCurrentValueSize(c->dbCursor, sz);
  }
}
RDF_Result 
RDF_CursorCurrentValueType (RDF_Cursor c, RDF_ValueType *type) {
  if (!c) {
    return RDF_NO_MORE_VALUES;
  } else {
    return DB_CursorCurrentValueType(c->dbCursor, (int*)type);
  }
}


RDF_Result
RDF_DisposeCursor(RDF_Cursor cursor) {
  DB_ReleaseCursor(cursor->dbCursor); 
  if (cursor->index) freeIndex(cursor->index);
  cursor->index = 0;
  freeCursor(cursor);
  return 0;
}


