/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public 
 * License version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Initial Developer of the Original Code is R.V.Guha. Portions
 * created by R.V.Guha are copyright (c) 2000 R.V.Guha. All Rights
 * Reserved.
 */

#include <string.h>
#include <stdio.h>
#include "rdf-int.h"
#include "xmlparse.h"
#include "contentSource.h"
#include "../utils/utils.h"
#include "../utils/hash.h"
#include "parsers.h"


/* The state of this parser is a sin against mankind and Guha should
   be sent to Gulag for this. If you make it conformant with the spec,
   God will forgive you for all your sins. */

#define RDFNS "http://www.w3.org/1999/02/22-rdf-syntax-ns#"
#define RDF_ABOUT "http://www.w3.org/1999/02/22-rdf-syntax-ns##about"
#define DMOZ_RDF_ABOUT "about"
#define RDF_ID "http://www.w3.org/1999/02/22-rdf-syntax-ns##ID"

#define DMOZ_RDF_ID "http://www.w3.org/TR/RDF/#id"

#define RDF_RESOURCE "http://www.w3.org/1999/02/22-rdf-syntax-ns##resource"
#define DMOZ_RDF_RESOURCE "http://www.w3.org/TR/RDF/#resource"
#define RDF_RDF "http://www.w3.org/1999/02/22-rdf-syntax-ns##RDF"
#define DMOZ_RDF_RDF "http://dmoz.org/rdf#RDF"
#define RDF_DESCRIPTION "http://www.w3.org/1999/02/22-rdf-syntax-ns##Description"
#define RDF_TYPE "http://www.w3.org/1999/02/22-rdf-syntax-ns##type"


#define RDF_CLASS "http://www.w3.org/2000/01/rdf-schema##Class"


char *
getAttributeValue (const char **attlist, const char *elName) {
  size_t n = 0;
  if (!attlist) return 0;
  while (*(attlist + n) != 0) {
    const char *attname = attlist[n];
    if (strcmp(attname, elName) == 0) return attlist[n + 1];
    n = n + 2;
  }
  return NULL;
}

char *
getHref(const char **attlist) {
  char *ans = getAttributeValue(attlist, RDF_RESOURCE);
  if (!ans) ans = getAttributeValue(attlist, DMOZ_RDF_RESOURCE); 
  if (!ans) ans = getAttributeValue(attlist, "resource"); 
  return ans;
}

void
addClass (const char* className, RDF_Parser f) {
  if (!HashLookup(f->classHash, className)) {
    char *cn = strdup(className);
    RDF_LoadFileAssert(f, f->db, cn, RDF_TYPE, RDF_CLASS, RDF_RESOURCE_TYPE);
    HashAdd(f->classHash, className, (void*)1);
  }
}
  
 
void
addElementProps (const char **attlist, const char* elementName, RDF_Parser f, RDF_Resource obj) {
  int count = 0;
  while (*(attlist + count) != 0) {
    const char* attName = attlist[count++];
    const char* attValue = attlist[count++];
    if (!streq(attName, RDF_ABOUT) && 
        !streq(attName, DMOZ_RDF_ABOUT) && 
        !streq(attName, RDF_RESOURCE) && 
        !streq(attName, DMOZ_RDF_RESOURCE) && 
        !streq(attName, DMOZ_RDF_ID) &&
        !streq(attName, RDF_ID)) {
      RDF_LoadFileAssert(f, f->db, obj, RDF_GetResource(attName), attValue,  RDF_STRING_TYPE);
    }
  }
}


void 
RDFXML_StartElementHandler (void* userData, const XML_Char *elementName, const XML_Char **attlist) { 
  RDF_Parser rdfp = (RDF_Parser) userData;      
  if (streq(elementName, RDF_RDF) || (streq(elementName, DMOZ_RDF_RDF))) {
    rdfp->status = EXPECTING_OBJECT;
    rdfp->depth++;
    return;
  }
  if ((rdfp->status != EXPECTING_OBJECT) && (rdfp->status != EXPECTING_PROPERTY)) return ;
  if (rdfp->status == EXPECTING_OBJECT) {
    char* url = NULL;
    char* xurl;
    RDF_Resource obj;
    int count = 0;    
    url = getAttributeValue(attlist, RDF_ABOUT);
    if (!url)  url = getAttributeValue(attlist, DMOZ_RDF_ABOUT);
    if (!url) {
      url = getAttributeValue(attlist, RDF_ID);
      if (!url) url =  getAttributeValue(attlist, DMOZ_RDF_ID);
      if (url) {
        xurl = getMem(strlen(url) + strlen(rdfp->url) + 2);
        strcat(xurl, rdfp->url);
        strcat(xurl, "#");
        strcat(xurl, url);
        url = xurl;
      }
    } else if (url[0] == '#') {
      xurl = getMem(strlen(url) + strlen(rdfp->url) + 2);
      strcat(xurl, rdfp->url);
      strcat(xurl, "#");
      strcat(xurl, url);
      url = xurl;
    }
    if (!url) {
      url = newAnonResource (rdfp, elementName) ;
      printf("\nNew anon resource = %s\n", url);
    } 
    obj =  RDF_GetResource(url);
    addElementProps (attlist, elementName, rdfp, obj) ;
    if (!streq(elementName, RDF_DESCRIPTION)) {
      RDF_Resource eln = RDF_GetResource(elementName);
      addClass(eln, rdfp) ;
      RDF_LoadFileAssert(rdfp, rdfp->db, obj, RDF_GetResource(RDF_TYPE),  eln, RDF_RESOURCE_TYPE);   
    }
    if (rdfp->depth > 1) {
      RDF_LoadFileAssert(rdfp, rdfp->db, rdfp->stack[rdfp->depth-2], 
                         rdfp->stack[rdfp->depth-1], obj, 
                         RDF_RESOURCE_TYPE);
    }
    rdfp->stack[rdfp->depth++] = strdup(obj);
    rdfp->status = EXPECTING_PROPERTY;
    
  } else if (rdfp->status == EXPECTING_PROPERTY) {
    char* url;
    RDF_Resource obj;
    int count = 0;
    url = getHref(attlist) ;      
    if (url && url[0] == '#') {
      char* xurl = getMem(strlen(url) + strlen(rdfp->url) + 2);
      strcat(xurl, rdfp->url);
      strcat(xurl, "#");
      strcat(xurl, url);
      url = xurl;
    }
    if (url) {
      RDF_Resource eln = RDF_GetResource(elementName);      
      obj =  RDF_GetResource(url);        
      addElementProps (attlist, elementName, rdfp, obj) ;     
      RDF_LoadFileAssert(rdfp, rdfp->db, rdfp->stack[rdfp->depth-1], eln, obj, RDF_RESOURCE_TYPE);
    } 
    rdfp->stack[rdfp->depth++] = RDF_GetResource(strdup(elementName));
    rdfp->status = EXPECTING_OBJECT;
  }
}

void
RDFXML_EndElementHandler (void *userData, const XML_Char *name) {
  RDF_Parser rdfp = (RDF_Parser) userData;          
  if ((rdfp->status != EXPECTING_OBJECT) && (rdfp->status != EXPECTING_PROPERTY)) {
    rdfp->cbuffLen = 0;
    if (rdfp->cbuff) rdfp->cbuff[0] = 0;
    return;
  }
    
  if ((rdfp->status == EXPECTING_OBJECT) && (rdfp->depth > 0)) {
    if (rdfp->cbuffLen) {
      RDF_Resource u = rdfp->stack[rdfp->depth-2];
      RDF_Resource s = rdfp->stack[rdfp->depth-1];
      RDF_LoadFileAssert(rdfp, rdfp->db, u, s, rdfp->cbuff , RDF_STRING_TYPE);  
    }
  } else  {
    /*    printf("Did not expect %s.\n Depth=%i %i\n", name, rdfp->depth, rdfp->status);   */
  } 
  if (rdfp->cbuff)   rdfp->cbuff[0] = 0;
  rdfp->cbuffLen = 0;
  if (rdfp->depth > 0) rdfp->depth--;
  rdfp->status = (rdfp->status == EXPECTING_OBJECT ? EXPECTING_PROPERTY : EXPECTING_OBJECT);
}

void 
RDFXML_CharacterDataHandler(void *userData, const XML_Char *s, int len) {
    RDF_Parser rdfp = (RDF_Parser) userData;   
  if (!nullString(s, len) && !rdfp->cbuffLen) {

    if (rdfp->cbuffLen + len + 1 > rdfp->cbuffSize) {
      rdfp->cbuffSize = rdfp->cbuffSize + (2 * len);
      rdfp->cbuff = realloc(rdfp->cbuff, rdfp->cbuffSize);
      rdfp->cbuff[rdfp->cbuffLen] = 0;
    }
    strncat(rdfp->cbuff, s, len);
    rdfp->cbuffLen = rdfp->cbuffLen + len;
  }
}

