/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public 
 * License version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Initial Developer of the Original Code is R.V.Guha. Portions
 * created by R.V.Guha are copyright (c) 2000 R.V.Guha. All Rights
 * Reserved.
 *
 * Contributors : Eric Van Der Vlist
 */


#include "../utils/utils.h"
#include "queryParser.h"
#include <stdio.h>
extern int       DB_Flush (const char* db) ;
extern int gVerbose;
extern int gAssertionCount;

int 
varp (Query q, const char* var) {
  if (var[0] != '?') {
    return 0;
  } else {
    Variable v = q->variables;
    while (v) {
      if (streq(v->name, var)) {
        return v->value == 0;
      }
      v = v->next;
    }
  }
  return 1;
}

const void*
vval (Query q, const char* var) {
  if (var[0] != '?') {
    return var;
  } else {    
    Variable v = q->variables;
    while (v) {
      if (streq(v->name, var)) {
        return v->value;
      }
      v = v->next;
    }
    return var;
  }
}
  
RDF_ValueType
vtype (Query q, const char* var) {
  if (var[0] != '?') {
    if (var[0] == '\'') {
      return RDF_STRING_TYPE;
    } else return RDF_RESOURCE_TYPE;
  } else {    
    Variable v = q->variables;
    while (v) {
      if (streq(v->name, var)) {
        return v->type;
      }
      v = v->next;
    }
    return 0;
  }
}
    

void
setVariableValue  (Query q, const char* var, void* value, RDF_ValueType type) {
  Variable v = q->variables;
  while (v) {
    if (streq(v->name, var)) {
      v->value = value;
      v->type  = type;
      break;
    }
    v = v->next;
  }
}


void
outputEscapedString (char* str, char* line) {
  char* vs ;
  *line++ = '\'';
  for (vs = strchr(str, '\''); vs != 0; vs = strchr(str, '\'')) {
    memcpy(line, str, vs-str);
    line += vs-str;
    *line++ = '\\';
    *line++ = '\'';
    str = vs+1;
  }
  memcpy(line, str, strlen(str));
  line += strlen(str);
  *line++ = '\\';
  *line++ = '\t';
  *line = 0;
}

int
addAnswer (Query q) {
  Variable v = q->variables;
  int l;
  if (q->outputType == TAB_LIMITED_OUTPUT && !q->queryHeaderPrinted) {
    Variable v1 = q->variables;
    q->queryHeaderPrinted = 1;
    while (v1) {
      if (v->outputVariablep) {
        write(q->cl->socket, v1->name, strlen(v1->name));
        write(q->cl->socket, "\t", 1);
      }
      v1 = v1->next;
    }
  if (q->cl->socket)  write(q->cl->socket, "\n", 1);
  }

  while (v) {
    if (v->value && v->outputVariablep) {
      char ansLine[512];
      if (v->type == RDF_RESOURCE_TYPE) {
        if (q->outputType == TAB_LIMITED_OUTPUT) {
          snprintf(ansLine, 512, "%s\t", (char*)v->value);
        } else {
          snprintf(ansLine, 512, "%s = %s\t", v->name, (char*)v->value);
        }
      } else if (v->type == RDF_STRING_TYPE) {
        if (strchr((char*)v->value, '\'')) {
          outputEscapedString((char*)v->value, ansLine);
        } else {
          if (q->outputType == TAB_LIMITED_OUTPUT) {
            snprintf(ansLine, 512, "'%s'\t", (char*)v->value);
          } else {
            snprintf(ansLine, 512, "%s = '%s'\t", v->name, (char*)v->value);
          }
        }
      } else if (v->type == RDF_INT_TYPE) {
        if (q->outputType == TAB_LIMITED_OUTPUT) 
          snprintf(ansLine, 512, "%i\t", (int)v->value);
        else 
          snprintf(ansLine, 512, "%s = %i\t", v->name, (int)v->value);
      }
      if (q->cl->socket) {
        write(q->cl->socket, ansLine, strlen(ansLine));
      } else return 0;
      if (gVerbose > 0) {
        printf("%s", ansLine);
      }
    }
    v = v->next;
  }
  if (gVerbose > 0) {
    printf("\n");
  }
  if (q->cl->socket)  write(q->cl->socket, "\n", 1);
  return 1;
}


int
SelectNextInt (Query q, Literal nl) {
  int err;
  if (q->abort) return 0;
  if (!nl) {
    if (!addAnswer(q)) {
      q->abort = 1;
      return 0;
    }
  } else if (!varp(q, nl->arc) && !varp(q, nl->source) && !varp(q, nl->target)) {
    if (0 != (err = RDF_HasAssertion(q->database, (const RDF_Resource) vval(q, nl->source), 
                                      (const RDF_Resource) vval(q, nl->arc), 
                                     vval(q, nl->target), vtype(q, nl->target)))) {
      if (err < 0) return err;
      SelectNextInt(q, nl->nxt);
    }
  } else {
    RDF_Cursor c = 0;
    Literal next;
    int len;
    char* var;
    if ((!varp(q, nl->arc)) && varp(q, nl->source) && (!varp(q, nl->target))) {
      var = nl->source;
      next = nl->nxt;
      err = RDF_GetSources(q->database, vval(q, nl->target), vval(q, nl->arc), &c);
    } else if ((!varp(q, nl->arc)) && !varp(q, nl->source) && (varp(q, nl->target))) {
      var = nl->target;
      next = nl->nxt;
      err = RDF_GetTargets(q->database, vval(q, nl->source), vval(q, nl->arc),  &c);
    } else if ((!varp(q, nl->arc)) && varp(q, nl->source) && varp(q, nl->target)) {
      var = nl->source;
      next = nl;
      err = RDF_GetArcSources(q->database, vval(q, nl->arc),  &c);
    } else if (varp(q, nl->arc) && (!varp(q, nl->source))) {
      var = nl->arc;
      next = (varp(q, nl->target) ? nl : nl->nxt);
      err = RDF_GetArcsOut(q->database, vval(q, nl->source),  &c);
    } else if (varp(q, nl->arc) && (!varp(q, nl->target))) {
      var = nl->arc;
      next = (varp(q, nl->source) ? nl : nl->nxt);
      err = RDF_GetArcsIn(q->database, vval(q, nl->target),  &c);
    } else return RDF_ERROR;
          
    if (err) return err;
    if (!c) return RDF_ERROR;
    
    while (0 == RDF_Advance(c)) {      
      void *value;
      RDF_ValueType type;
      RDF_CursorCurrentValue(c , &value);
      RDF_CursorCurrentValueSize(c , &len);
      RDF_CursorCurrentValueType(c, &type);
      if (type == RDF_RESOURCE_TYPE || type == RDF_STRING_TYPE) ((char*)value)[len] = 0; 
      setVariableValue(q, var, value, type);
      err = SelectNextInt(q, next);
      if (err) {
        RDF_DisposeCursor(c);
        return err;
      }
    }
    setVariableValue(q, var, 0, 0);
    RDF_DisposeCursor(c);  
  }
  return 0;
}


char*
appendNamespace (char* str, Namespace ns) {
 char* st =  strchr(str, ':');
 if (!st) {
   if (strlen(ns->prefix) > 0) {
     return strdup(str);
   } else {
     char* nn =  getMem(strlen(str) + strlen(ns->url) + 1);
     memcpy(nn, ns->url, strlen(ns->url));
     strcat(nn, "#");
     strcat(nn, str);
     return nn;
   }
 } else {
   char* nn =  getMem(strlen(st+1) + strlen(ns->url) + 1);
   memcpy(nn, ns->url, strlen(ns->url));
   strcat(nn, "#");
   strcat(nn, st+1);
   return nn;
 }
}

int
namespaceMatch(char* uri, Namespace ns) {
  if (strlen(ns->prefix) == 0) {
    return (strchr(uri, ':') == 0);
  } else {
    char* st = strchr(uri, ':');
    if (!st) return 0;
    if ((st-uri) != strlen(ns->prefix)) return 0;
    return (strncmp(uri, ns->prefix, st-uri) == 0);
  }
}

void
namespaceMungeLiteral (Namespace ns, Literal l) {
  if (l->source[0] != '?') {
    Namespace nsx = ns;
    while (nsx) {
      if (namespaceMatch(l->source, nsx)) {
        char *nn = appendNamespace(l->source, nsx);
        freeMem(l->source);
        l->source = nn;
        break;
      }
      nsx = nsx->next;
    }
  }
  if (l->type == RDF_RESOURCE_TYPE) {
    char* trg = (char*) l->target;
    if (trg[0] != '?') {
      Namespace nsx = ns;
      while (nsx) {
        if (namespaceMatch(trg, nsx)) {
          char *nn = appendNamespace(trg, nsx);
          freeMem(l->target);
          l->target = nn;
          break;
        }
        nsx = nsx->next;
      }
    }        
  }
  {
    char* arc = (char*) l->arc;
    Namespace nsx = ns;
    while (nsx) {
      if (namespaceMatch(arc, nsx)) {
        char *nn = appendNamespace(arc, nsx);
        freeMem(l->arc);
        l->arc = nn;
        break;
      }
      nsx = nsx->next;
    }
  }        
}

Literal
orderLiterals1 (Query q) {
  Literal l, n;
  n = 0;
  for (l = q->literals; l != 0; l = l->next) {
    l->score = 0;
    if (varp(q, l->arc)) l->score++;
    if (varp(q, l->source)) l->score++;
    if (varp(q, l->target)) l->score++;
  }
  for (l = q->literals; l != 0; l = l->next) {
    if (!l->taken) {
      n = l; break;
    }
  }
  for (l = q->literals; l != 0; l = l->next) {
    if (!l->taken && l->score < n->score) 
      n = l;
  }
  return n;
}

void
orderLiterals (Query q) {
  Literal  next, prev;
  for (next =  orderLiterals1(q); next != 0; next = orderLiterals1(q)) {
    if (!q->lits) {
      q->lits = next;
    } else {
      prev->nxt = next;
    }
    if (varp(q, next->source)) {
      setVariableValue(q, next->source, "temp", RDF_STRING_TYPE);
    }
    if (varp(q, next->target)) {
      setVariableValue(q, next->target, "temp", RDF_STRING_TYPE);
    }
    next->taken = 1;
    prev = next;
  }    
  {
    Variable v ;
    for (v = q->variables; v != 0; v = v->next) {
      v->value = 0;
    }
  }
}


int
processQueryInt (const char* queryString, Query *qpt, ClientConnection cl) {
  QueryParseResult err;
  Query            query;
  Literal l ;

  if (gVerbose > 0) {
    time_t tm ;
    tm = time(&tm);
    printf("%sQuery> %s\n", ctime(&tm), queryString);
    fflush(stdout);
  }


  if (0 != (err = parseQuery(queryString, qpt))) {
    return err;
  }


  query = *qpt;

  query->cl = cl;

  if (cl->namespaces) {
    for (l = query->literals; l != 0; l = l->next) {
      namespaceMungeLiteral(cl->namespaces, l);
    }
  }


  if (gVerbose > 2) {
    describeQuery (query) ;
  }

  switch (query->type) {

  case SELECT :
    {
      orderLiterals(query);
      if (0 != (err = SelectNextInt(query, query->lits))) {
        freeQuery(query);
        query = 0;
        return err;
      }
      break;
    }

  case UPLOAD :
    {
      char* code = 0;
      int start = gAssertionCount;
      err = RDF_LoadURLIntoDB (query->database, query->url, query->fileType, 1, &code);
      DB_Flush (query->database) ;
      printf("%i Assertions Added\n", gAssertionCount-start);
      if (code) {
        write(cl->socket, code, strlen(code));
        write(cl->socket, "\n", 1);
      }
      break;
    }

  case UNLOAD :
    {
      err = RDF_UnLoadURLFromDB (query->database, query->url);
      DB_Flush (query->database) ;
      break;
    }

  case CREATE :
    {
      RDF_DB db;
      err = RDF_GetDB (query->database, 1, &db);
      break;
    }
    
  case INSERT :
    {
      Literal l = query->literals;
      while (l) {
        err = RDF_Assert(query->database, l->source, l->arc, l->target, l->type);
        l = l->next;
      }
      DB_Flush (query->database) ;
      break;
    }

  case DELETE :    
    {
      Literal l = query->literals;
      while (l) {
        err = RDF_Unassert(query->database, l->source, l->arc, l->target, l->type);
        l = l->next;
      }
      DB_Flush (query->database) ;
      break;
    }

  case ENTER :    
    {
      char* prefix = query->namespacePrefix;
      if (startsWith(prefix, "xmlns:")) {
        addNamespace(cl, &prefix[6], query->namespace);
      } else {
        addNamespace(cl, "", query->namespace);
      }
      break;
    }

  case LEAVE :    
    {
      char* prefix = query->namespacePrefix;
      if (startsWith(prefix, "xmlns:")) {
        removeNamespace(cl, &prefix[6], query->namespace);
      } else {
        removeNamespace(cl, "", query->namespace);
      }
      break;
    }

  }
  
  /*   write(fd, "%i", q->id); */
  if (err) {
    return err;
  } else {
    return query->id;
  }
}

#define HELP_MSG "Please look at http://web1.guha.com/rdfdb/ for help"

int
ProcessQuery (const char* query, ClientConnection cl) {
  int fd = cl->socket;
  Query q = 0;
  int qid;
  char errStr[32];

  if (streq(query, "help")) {
    writen(fd, HELP_MSG, strlen(HELP_MSG));
    return 0;
  }

  qid = processQueryInt(query, &q, cl);

  if (cl->socket) {
    if (qid < 0) {
      snprintf(errStr, 32, "%i </>\n", qid);
      writen(fd, errStr, strlen(errStr));
    } else {
      writen(fd, "0 </>\n", strlen("0 </>\n"));
    }  
  }

  if (q && !qid) freeQuery(q);
  

}
