/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public 
 * License version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Initial Developer of the Original Code is R.V.Guha. Portions
 * created by R.V.Guha are copyright (c) 2000 R.V.Guha. All Rights
 * Reserved.
 */

#include "../librdf/rdf.h"
#include "../utils/utils.h"
#include "../server/connection.h"

#include <stdio.h>
typedef enum {
  NO_PARSE_ERROR = 0,
  UNKNOWN_QUERY_TYPE_ERROR = -1,
  GENERAL_SYNTAX_ERROR = -2,
  MALFORMED_LITERAL_ERROR = -3
} QueryParseResult;

typedef enum {
  UNKNOWN_QUERY_TYPE = 0,
  SELECT,
  UPLOAD,
  UNLOAD,
  ENTER,
  LEAVE,
  INSERT,
  DELETE,
  CREATE,
  DROP
} QueryType;

typedef enum {
  RDF_XML_OUTPUT,
  JAVASCRIPT_OUTPUT,
  TAB_LIMITED_OUTPUT,
  VAR_LIST_OUTPUT
} QueryOutputFormat;
     

typedef struct _LiteralStruct {
  RDF_Resource source;
  RDF_Resource arc;
  void* target;
  RDF_ValueType type;
  int taken;
  int score;
  struct _LiteralStruct* next;
  struct _LiteralStruct* nxt;
} LiteralStruct;

typedef LiteralStruct* Literal;

typedef struct _VariableStruct {
  int   outputVariablep;
  int   verified;
  char* name;
  void* value;
  int   size;
  RDF_ValueType type;
  struct _VariableStruct* next;
} VariableStruct;

typedef VariableStruct* Variable;

typedef struct _QueryStruct {
  int id;
  char* queryString;
  char* database;
  QueryType type;
  Literal literals;
  Literal lits;
  Variable variables;
  char* namespacePrefix;
  char* namespace;
  char* url;
  RDF_Cursor c;
  RDF_SerializationType fileType;
  char** ans;
  int outputInited;
  QueryOutputFormat outputType;
  int abort;
  ClientConnection cl;
  int queryHeaderPrinted;
} QueryStruct;

typedef QueryStruct *Query;

typedef QueryParseResult (*QueryParserFunc)(Query nq);

QueryParseResult parseQuery (const char* queryString, Query *q);
void  freeQuery (Query q);
int ProcessQuery (const char* query,  ClientConnection cl) ;
