/*
 * Example of server using TCP protocol.
 */


#include	<stdio.h>
#include	<sys/types.h>
#include	<sys/socket.h>
#include	<netinet/in.h>
#include	<arpa/inet.h>
#include        "../utils/utils.h"
#include        "connection.h"


#define MAXLINE 512
#include <pthread.h>
#include <errno.h>
#include <sys/types.h>
#include <signal.h>
#define QUERY_END "</>"
#define RESPONSE_END  "\n</>\n"


pthread_key_t gCCKey;

int
clientSocket (ClientConnection cl) {
  return cl->socket;
}

int 
catchBrokenPipe (int sig) {
  ClientConnection conn =  pthread_getspecific(gCCKey);
  conn->socket = 0;
  printf("Catching broken pipe signal\n");
  signal(SIGPIPE, catchBrokenPipe);
}

void
addNamespace (ClientConnection cl, char *prefix, char* url) {
  Namespace ns = (Namespace) getMem(sizeof(NamespaceStruct));
  ns->prefix = strdup(prefix);
  ns->url    = strdup(url);
  ns->next = cl->namespaces;
  cl->namespaces = ns;
}

void
freeNamespace (Namespace ns) {
  freeMem(ns->prefix);
  freeMem(ns->url);
  freeMem(ns);
}

void removeNamespace (ClientConnection cl, char* prefix) {
  Namespace next = cl->namespaces;
  Namespace prev = next;
  while (next) {
    if (streq(prefix, next->prefix)) {
      if (prev == next) {
	cl->namespaces = next->next;
      } else {
	prev->next = next->next;
      }
      freeNamespace(next);
      break;
    }
    prev = next;
    next = next->next;
  }
}

pthread_mutex_t gConnectionListMutex = PTHREAD_MUTEX_INITIALIZER;
ClientConnection gAllClientConnections = 0;

void
terminateConnection (ClientConnection conn) {
  pthread_mutex_lock(&gConnectionListMutex);
  {
    ClientConnection all = gAllClientConnections;
    ClientConnection prev = gAllClientConnections;
    if (conn->socket) {
      close(conn->socket);
      conn->socket = 0;
    }
    while (all) {
      if (conn == all) {
	Namespace ns = conn->namespaces;
	Namespace next ;
	if (prev == all) {
	  gAllClientConnections = all->next;
	} else {
	  prev->next = all->next;
	}
	while (ns) {
	  next = ns->next;
	  freeNamespace(ns);
	  ns = next;
	}
	freeMem((void*)all);
	break;
      }
      prev = all;
      all = all->next;
    }
  }
  pthread_mutex_unlock(&gConnectionListMutex);
  printf("terminated connection\n");
}
    
void
terminateAllConnections () {
  pthread_mutex_lock(&gConnectionListMutex);
  {
    ClientConnection rest = gAllClientConnections;
    
    while (rest) {
      ClientConnection next = rest->next;
      close(rest->socket);
      rest->socket = 0;
      freeMem((void*)rest);
      rest = next;
    }
    
    gAllClientConnections = 0;
  }
  pthread_mutex_unlock(&gConnectionListMutex);
}

ClientConnection
addConnection (int sockfd) {
  ClientConnection newcon = (ClientConnection) getMem(sizeof(ClientConnectionStruct)) ;
  newcon->socket = sockfd;
  pthread_mutex_lock(&gConnectionListMutex);
  newcon->next = gAllClientConnections;
  gAllClientConnections = newcon;
  pthread_mutex_unlock(&gConnectionListMutex);
  return newcon;
}

void*
pc (void* conn) {
}
  
void*
processConnection (void* conn) {
  ClientConnection connec  = (ClientConnection) conn;
  int   fd = connec->socket;
  int	n;
  char	line[MAXLINE];
  char  query[4 * MAXLINE];
  char* rline;
  int   offset = 0;
  int   err = 0;

  signal(SIGPIPE, catchBrokenPipe);
  pthread_setspecific(gCCKey, conn);
  for ( ; ; ) {
    bzero(query, 4 * MAXLINE);
    bzero(line, MAXLINE);
  
    n = readline((int)fd, line, MAXLINE);
    rline = line;
    if (n == 0) {
      terminateConnection(connec);
      return;		/* connection terminated */
    } 

    if (n < 0) {
      printf("str_echo: readline error");
      terminateConnection(connec);
      return;
    } 

    rline = trimLine(rline);

    if (streq(rline, "<quit>")) {
      printf("Closing connection\n");
      terminateConnection(connec);
      return;
    } 


    memcpy(query, rline, strlen(rline));
    if (endsWith(rline, QUERY_END)) {
      char* rest = strstr(&query[strlen(query)-5], QUERY_END);
      if (rest) *rest = 0;
      ProcessQuery(query, connec);
      printf("\nDone\n");
    } else {
      strcat(query, " ");
      for (;;) {
	n = readline((int)fd, line, MAXLINE);
	rline = line;
	rline = trimLine(rline);
	if (n == 0) {
	  terminateConnection(connec);
	  return;		/* connection terminated */
	} else if (n < 0) {
	  printf("str_echo: readline error");
	  terminateConnection(connec);
	  return;
	} else {
	  strcat(query, rline);
	  /*	  memcpy(&query[offset], rline, strlen(rline)); */

	  if (endsWith(rline, QUERY_END)) {
	    char* rest = strstr(&query[strlen(query)-5], QUERY_END);
	    if (rest) *rest = 0; 
	    ProcessQuery(query, connec);
	    printf("\nDone\n");
	    break;
	  } else {
	    strcat(query, " ");
	  }
	}
      }
    }
  }
}


int
getServerPort () {
  char *port = getenv("RDFDB_PORT");
  if (port) {
    int ip = atoi(port);
    if (ip) {
      printf("Binding = %i\n", ip);
      return ip;
    } else {
      return 7001;
    }
  } else {
    printf("\nPlease set the server port by setting the environment variable RDFDB_PORT\n.e.g., export RDFDB_PORT=7001\n");
    exit(1);
  }  
}

void CloseAllDatabases () ;
static int gListenSockfd = 0;
int closeAndExit (int sig) {
  if (sig == SIGINT) {
    close(gListenSockfd);
    terminateAllConnections();
    CloseAllDatabases () ;
    printf("Exiting on SIGINT ...\n");
    exit(0);
  }
}


int
openService() {
  int			sockfd, newsockfd, clilen, childpid;
  struct sockaddr_in	cli_addr, serv_addr;
  pthread_t thread;
  int err;
  pthread_attr_t attr;

  /* Open a TCP socket */
  
  if ((gListenSockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
    printf("server: can't open stream socket");
    return -1;
  }
  
  /*  Bind our local address so that the client can send to us. */
  
  bzero((char *) &serv_addr, sizeof(serv_addr));
  serv_addr.sin_family      = AF_INET;
  serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);
  serv_addr.sin_port        = htons(getServerPort());
  
  printf("Binding local socket ...");
  fflush(stdout);
  while (bind(gListenSockfd, (struct sockaddr *) &serv_addr, sizeof(serv_addr)) < 0) {
    sleep(1);
  }
  printf("done\n");

  signal(SIGINT, closeAndExit);  

  listen(gListenSockfd, 128);

  pthread_key_create(&gCCKey, 0);

  pthread_attr_init(&attr);
  pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
  for ( ; ; ) {
    /* Wait for a connection from a client process. */    
    clilen = sizeof(cli_addr);
    printf("Awaiting connections ...\n");
    newsockfd = accept(gListenSockfd, (struct sockaddr *) &cli_addr,  &clilen);
    if (newsockfd < 0) {
      printf("server: accept error\n");
    } else {      
      ClientConnection cl = addConnection(newsockfd);      
      err = pthread_create(&thread, 0, processConnection, (void*)cl);
      if (err) {
	printf("Problems spawning thread!\n");
      }
    }
  }
}
