/*
 * Example of server using TCP protocol.
 */

#include	"inet.h"

#define MAXLINE 512
#include <pthread.h>
#include <errno.h>
#include <sys/types.h>
#include <signal.h>
int
writen(fd, ptr, nbytes)
register int	fd;
register char	*ptr;
register int	nbytes;
{
	int	nleft, nwritten;

	nleft = nbytes;
	while (nleft > 0) {
		nwritten = write(fd, ptr, nleft);
		if (nwritten <= 0)
			return(nwritten);		/* error */

		nleft -= nwritten;
		ptr   += nwritten;
	}
	return(nbytes - nleft);
}

/*
 * Read a line from a descriptor.  Read the line one byte at a time,
 * looking for the newline.  We store the newline in the buffer,
 * then follow it with a null (the same as fgets(3)).
 * We return the number of characters up to, but not including,
 * the null (the same as strlen(3)).
 */

int testInt = 0;

int
readline(fd, ptr, maxlen)
register int	fd;
register char	*ptr;
register int	maxlen;
{
	int	n, rc;
	char	c;

	for (n = 1; n < maxlen; n++) {
		if ( (rc = read(fd, &c, 1)) == 1) {
			*ptr++ = c;
			if (c == '\n')
				break;
		} else if (rc == 0) {
			if (n == 1)
				return(0);	/* EOF, no data read */
			else
				break;		/* EOF, some data was read */
		} else
			return(-1);	/* error */
	}

	*ptr = 0;
	return(n);
}

str_echo(sockfd)
int	sockfd;
{
	int	n;
	char	line[MAXLINE];
	printf("And here\n");
	for ( ; ; ) {
	  bzero(line, MAXLINE);
		n = readline(sockfd, line, MAXLINE);
		if (n == 0)
			return;		/* connection terminated */
		else if (n < 0)
			printf("str_echo: readline error");
		printf("%i)%s\n", testInt++, line);
		if (writen(sockfd, line, n) != n)
			printf("str_echo: writen error");
	}
}

void*
processConnection (void* fd) {
  printf("Got here\n");
  str_echo((int)fd);
}

main(argc, argv)
int	argc;
char	*argv[];
{
	int			sockfd, newsockfd, clilen, childpid;
	struct sockaddr_in	cli_addr, serv_addr;
	pthread_t thread;
	pname = argv[0];

	/*
	 * Open a TCP socket (an Internet stream socket).
	 */

	if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
	  printf("server: can't open stream socket");
	  exit(1);
	}

	/*
	 * Bind our local address so that the client can send to us.
	 */
	
	bzero((char *) &serv_addr, sizeof(serv_addr));
	serv_addr.sin_family      = AF_INET;
	serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);
	serv_addr.sin_port        = htons(6004);

	printf("Binding local socket\n");
	while (bind(sockfd, (struct sockaddr *) &serv_addr, sizeof(serv_addr)) < 0) {
	}
	printf("done\n");
	listen(sockfd, 5);

	for ( ; ; ) {
	  int err;
		/*
		 * Wait for a connection from a client process.
		 * This is an example of a concurrent server.
		 */

		clilen = sizeof(cli_addr);
		newsockfd = accept(sockfd, (struct sockaddr *) &cli_addr,
								     &clilen);
		if (newsockfd < 0) {
			printf("server: accept error");
			exit(1);
		}


		
		err = pthread_create(&thread, NULL, processConnection, (void*)newsockfd);
		printf("Creation %i\n", err);
	}
}
