/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public 
 * License version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Initial Developer of the Original Code is R.V.Guha. Portions
 * created by R.V.Guha are copyright (c) 2000 R.V.Guha. All Rights
 * Reserved.
 */

#include "hash.h"
#include "utils.h"

int 
hashKey (HashTable ht, const char* key) {
  size_t len = strlen(key);
  int    sum = 0;
  size_t    n = 0;
  int ans;
  for (n = 0; n < len; n++) sum = sum + (int)key[n];
  ans = sum & ht->size;
  if (ans == ht->size) ans = ans-1;
  return ans; 
}

HashTable 
NewHashTable(int size) {
  HashTable ht = (HashTable)getMem(sizeof(HashTableStruct));
  ht->size = size;
  ht->buckets = (HashEntry*)getMem(sizeof(HashEntry) * size);
  return ht;
}

void*
HashLookup(HashTable ht, const char* key) {
  int offset = hashKey(ht, key);
  HashEntry he = ht->buckets[offset];
  while (he) {
    if (strcmp(he->key, key) == 0) return he->value;
    he = he->next;
  }
  return NULL;
}

void 
HashAdd (HashTable ht, const char* key, void* value) {
  int offset = hashKey(ht, key);
  HashEntry he = ht->buckets[offset];
  HashEntry prev = he;
  char* ckey;
  while (he) {
    if (strcmp(he->key, key) == 0) {
      if (value == he->value) {
	return;
      } else {
	he->value = value;
	return;
      }
    }
    prev = he;
    he = he->next;
  }
  he = (HashEntry) getMem(sizeof(HashEntryStruct));
  he->value = value;
  he->key   = strdup(key);
  if (prev) {
    prev->next = he;
  } else {
    ht->buckets[offset] = he;
  }
}
 
