/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public 
 * License version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Initial Developer of the Original Code is R.V.Guha. Portions
 * created by R.V.Guha are copyright (c) 2000 R.V.Guha. All Rights
 * Reserved.
 */

#include "utils.h"

char* 
getMem (size_t l) {
  char* ans = malloc(l);
  if (ans) bzero(ans, l);
  return ans;
}

void
freeMem (char* mem) {
  free((void*)mem); 
}

char* 
trimLine (char* line) {
  size_t len = strlen(line);
  size_t index = len-1;
  while (index > 0) {
    if (isspace(line[index])) {
      line[index] = 0;
      index--;
    } else break;
  }
  index = 0;
  while (index < len) {
    if (isspace(line[index])) {
      index++;
    } else break;
  }
  return &line[index];
}

int
nullString (const char* line, int len) {
  int n = 0;
  while (n < len) {
    if (!isspace(line[n])) return 0;
    n++;
  } 
  return 1;
}


int
startsWith (const char* str, const char* prefix) {
  size_t l1 = strlen(str);
  size_t l2 = strlen(prefix);
  if (l1 >= l2) {
    return (strncasecmp(str, prefix, l2) == 0);
  } else return 0;
}

int
endsWith (const char* str, const char* suffix) {
  size_t l1 = strlen(str);
  size_t l2 = strlen(suffix);

  if (l1 >= l2) {
    return (strncasecmp(&str[l1-l2], suffix, l2) == 0);
  } else return 0;
}


int
writen(int fd, char *ptr, int nbytes) {
  int	nleft, nwritten;
  
  nleft = nbytes;
  while (nleft > 0) {
    nwritten = write(fd, ptr, nleft);
    if (nwritten <= 0)
      return(nwritten);		/* error */
    
    nleft -= nwritten;
    ptr   += nwritten;
  }
  return(nbytes - nleft);
}

/*
 * Read a line from a descriptor.  Read the line one byte at a time,
 * looking for the newline.  We store the newline in the buffer,
 * then follow it with a null (the same as fgets(3)).
 * We return the number of characters up to, but not including,
 * the null (the same as strlen(3)).
 */

int
readline(int fd, char *ptr, int maxlen) {
  int	n, rc;
  char	c;
  
  for (n = 1; n < maxlen; n++) {
    if ( (rc = read(fd, &c, 1)) == 1) {
      *ptr++ = c;
      if (c == '\n')         
        break;
    } else if (rc == 0) {
      if (n == 1)
        return(0);	/* EOF, no data read */
      else
        break;		/* EOF, some data was read */
    } else
      return(-1);	/* error */
  }
  
  *ptr = 0;
  return(n);
}

/* Read upto maxlen bytes off the fd */

int
readn(int fd, char *ptr, int maxlen) {
  int	n, rc;
  char	c;
  
  bzero(ptr, maxlen);
  for (n = 1; n < maxlen; n++) {
    if ((rc = read(fd, ptr, 1)) == 1) {
      /*      *ptr++ = c; */
    } else if (rc == 0) {
      if (n == 1)
        return(0);	/* EOF, no data read */
      else
        break;		/* EOF, some data was read */
    } else
      return(-1);	/* error */
  }
  
  /*  *ptr = 0; */
  return(n);
}

